/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.weakforced.osgi;

import java.util.List;
import com.openexchange.antiabuse.AntiAbuseService;
import com.openexchange.config.ConfigurationService;
import com.openexchange.exception.OXException;
import com.openexchange.java.Strings;
import com.openexchange.login.listener.LoginListener;
import com.openexchange.osgi.HousekeepingActivator;
import com.openexchange.rest.client.endpointpool.EndpointManagerFactory;
import com.openexchange.rest.client.httpclient.HttpClientService;
import com.openexchange.rest.client.httpclient.SpecificHttpClientConfigProvider;
import com.openexchange.timer.TimerService;
import com.openexchange.version.VersionService;
import com.openexchange.weakforced.WeakforcedClient;
import com.openexchange.weakforced.WeakforcedEndpointManager;
import com.openexchange.weakforced.WeakforcedLoginListener;
import com.openexchange.weakforced.WeakforcedProperties;


/**
 * {@link WeakforcedActivator}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v1.0.0
 */
public class WeakforcedActivator extends HousekeepingActivator {

    private WeakforcedEndpointManager endpointManager;

    /**
     * Initializes a new {@link WeakforcedActivator}.
     */
    public WeakforcedActivator() {
        super();
    }

    @Override
    protected Class<?>[] getNeededServices() {
        return new Class<?>[] { ConfigurationService.class, EndpointManagerFactory.class, TimerService.class, VersionService.class };
    }

    @Override
    protected synchronized void startBundle() throws Exception {
        org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(WeakforcedActivator.class);

        trackService(HttpClientService.class);
        openTrackers();

        ConfigurationService configService = getService(ConfigurationService.class);

        String secret = configService.getProperty(WeakforcedProperties.HASH_SECRET);
        if (Strings.isEmpty(secret)) {
            throw OXException.general("Denied initialization due to unset secret configuration. Please specify property 'com.openexchange.weakforced.hash.secret'.");
        }

        String algorithm = configService.getProperty(WeakforcedProperties.HASH_ALGORITHM);
        if (Strings.isEmpty(secret)) {
            algorithm = "SHA-256";
        } else {
            algorithm = Strings.toUpperCase(algorithm.trim());
        }

        boolean truncate = configService.getBoolProperty(WeakforcedProperties.HASH_TRUNCATE, true);

        EndpointManagerFactory factory = getService(EndpointManagerFactory.class);
        WeakforcedEndpointManager endpointManager = new WeakforcedEndpointManager(this);
        List<SpecificHttpClientConfigProvider> configProviders = endpointManager.init(factory, this);
        for (SpecificHttpClientConfigProvider configProvider : configProviders) {
            registerService(SpecificHttpClientConfigProvider.class, configProvider);
        }
        this.endpointManager = endpointManager;

        String[] attributeNames;
        {
            String sAttributes = configService.getProperty(WeakforcedProperties.ATTRIBUTES);
            if (Strings.isEmpty(sAttributes)) {
                attributeNames = null;
            } else {
                attributeNames = Strings.splitByComma(sAttributes);
            }
        }

        WeakforcedClient weakforcedClient = new WeakforcedClient(endpointManager, secret, algorithm, truncate);
        registerService(AntiAbuseService.class, weakforcedClient);

        registerService(LoginListener.class, new WeakforcedLoginListener(configService, weakforcedClient, attributeNames));
        logger.info("Started bundle: {}", context.getBundle().getSymbolicName());
    }

    @Override
    protected synchronized void stopBundle() throws Exception {
        WeakforcedEndpointManager endpointManager = this.endpointManager;
        if (null != endpointManager) {
            this.endpointManager = null;
            endpointManager.shutDown();
        }
        super.stopBundle();

        org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(WeakforcedActivator.class);
        logger.info("Stopped bundle: {}", context.getBundle().getSymbolicName());
    }

}
